{% extends "basic/layout.html" %}

{% block rootrellink %}
        <li><a href="{{ pathto('index') }}">home</a>|&nbsp;</li>
        <li><a href="{{ pathto('whatis') }}">what is globus provision?</a>|&nbsp;</li>
        <li><a href="{{ pathto('ami') }}">AMIs</a>|&nbsp;</li>
        <li><a href="{{ pathto('download') }}">download</a>|&nbsp;</li>
        <li><a href="{{ pathto('support') }}">support</a>|&nbsp;</li>
       <li><a href="{{ pathto('docs') }}">documentation </a> &raquo;</li>
{% endblock %}

{% block relbar1 %}

<div style="background-color: white; text-align: left; padding: 10px 10px 15px 35px">
<a href="{{ pathto('index') }}"><img src="{{
pathto("_static/logo.png", 1) }}" border="0" alt="Globus Provision"/></a>
</div>
{{ super() }}
{% endblock %}


{# put the sidebar before the body #}
{% block sidebar1 %}{{ sidebar() }}{% endblock %}
{% block sidebar2 %}{% endblock %}

{%- block extrahead %}
{{ super() }}
<script type="text/javascript">

  var _gaq = _gaq || [];
  _gaq.push(['_setAccount', 'UA-4920033-5']);
  _gaq.push(['_trackPageview']);

  (function() {
    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
  })();

</script>
{% endblock %}
