##  What machine is your central manager?

CONDOR_HOST = <%= @server %>


##  Internet domain of machines sharing a common UID space.

UID_DOMAIN = <%= @domain %>


##  Internet domain of machines sharing a common file system.

FILESYSTEM_DOMAIN = <%= @domain %>


## Pool's short description

COLLECTOR_NAME = DemoGrid Condor Pool


##  When is this machine willing to start a job? 

START = TRUE


##  When to suspend a job?

SUSPEND = FALSE


##  When to nicely stop a job?
##  (as opposed to killing it instantaneously)

PREEMPT = FALSE


##  When to instantaneously kill a preempting job
##  (e.g. if a job is in the pre-empting stage for too long)

KILL = FALSE


##  This macro determines what daemons the condor_master will start and keep its watchful eyes on.
##  The list is a comma or space separated list of subsystem names

DAEMON_LIST = <%= @daemons %>

ALLOW_READ = *.<%= @domain %>, 127.0.*
ALLOW_WRITE = *.<%= @domain %>, 127.0.*
